@@ -67,12 +67,12 @@ def load_patterns(h5_file, features=None):
         Each type of pattern contains a field for each loaded feature.
     """
     with h5py.File(h5_file, 'r') as f:
-        patterns = {'items': f['items'][()],
+        patterns = {'items': [item for item in f['items'].asstr()],
                     'vector': {},
                     'similarity': {}}
 
         if features is None:
-            features = f['features'][()]
+            features = f['features'].asstr()
 
         for name in features:
             patterns['vector'][name] = f['vector/' + name][()]
@@ -168,6 +168,31 @@ def init_plot(**kwargs):
 
 
 class LayerIndex(object):
+    """
+    Representation of one layer of a network.
+
+    Parameters
+    ----------
+    layer_segments : dict of (str: dict of (str: int))
+        Size of each named segment for each sublayer.
+
+    Attributes
+    ----------
+    size : int
+        Number of units in the layer.
+
+    size_sublayer : dict of (str: int)
+        Number of units in each sublayer.
+
+    size_segment : dict of (str: dict of (str: int))
+        Size of each segment in each sublayer.
+
+    sublayer : dict of (str: numpy.ndarray)
+        Start and stop units of each sublayer.
+
+    segment : dict of (str: dict of (str: numpy.ndarray)
+        Start and stop units of each segment.
+    """
     def __init__(self, layer_segments):
         self.size = 0
         self.size_sublayer = {}
@@ -206,6 +231,10 @@ class LayerIndex(object):
     def get_segment(self, sublayer, segment):
         return self.segment[sublayer][segment]
 
+    def get_slice(self, sublayer, segment):
+        ind = self.get_segment(sublayer, segment)
+        return slice(ind[0], ind[1])
+
     def get_unit(self, sublayer, segment, index):
         return self.segment[sublayer][segment][0] + index
 
@@ -307,30 +336,62 @@ class Network(object):
         self.w_ff_exp[:] = 0
         self.w_ff_pre[:] = 0
 
-    def copy(self):
+    def copy(self, include=None, exclude=None):
         """
         Copy the network to a new network object.
 
+        Parameters
+        ----------
+        include : list of str, optional
+            List of fields to include. Default is a standard list of
+            fields.
+
+        exclude : list of str, optional
+            List of fields to exclude. Overrides values for include.
+
         Returns
         -------
         net : cymr.Network
             Network with the same segments, weights, and activations.
         """
+        # all attributes that contain an array
+        fields = [
+            'f', 'f_in', 'c', 'c_in', 'w_fc_exp', 'w_fc_pre',
+            'w_cf_exp', 'w_cf_pre', 'w_ff_exp', 'w_ff_pre'
+        ]
+
+        # set the fields to include in the copy (default is all)
+        if include is None:
+            include = fields
+        if exclude is not None:
+            include = [i for i in include if i not in exclude]
+
+        # copy included attributes
         net = Network(self.f_segment, self.c_segment)
-        net.f = self.f.copy()
-        net.f_in = self.f_in.copy()
-        net.c = self.c.copy()
-        net.c_in = self.c_in.copy()
-        net.w_fc_exp = self.w_fc_exp.copy()
-        net.w_fc_pre = self.w_fc_pre.copy()
-        net.w_cf_exp = self.w_cf_exp.copy()
-        net.w_cf_pre = self.w_cf_pre.copy()
-        net.w_ff_exp = self.w_ff_exp.copy()
-        net.w_ff_pre = self.w_ff_pre.copy()
+        for field in fields:
+            if field in include:
+                setattr(net, field, getattr(self, field).copy())
+            else:
+                delattr(net, field)
         return net
 
     def get_sublayer(self, layer, sublayer):
-        """Get indices for a sublayer."""
+        """
+        Get indices for a sublayer.
+
+        Parameters
+        ----------
+        layer : str
+            Layer to access.
+
+        sublayer : str
+            Sublayer to access.
+
+        Returns
+        -------
+        numpy.ndarray
+            Start and stop indices for the sublayer.
+        """
         if layer == 'f':
             ind = self.f_ind.get_sublayer(sublayer)
         elif layer == 'c':
@@ -340,7 +401,23 @@ class Network(object):
         return ind
 
     def get_sublayers(self, layer, sublayers):
-        """Get an array of indices for multiple sublayers."""
+        """
+        Get an array of indices for multiple sublayers.
+
+        Parameters
+        ----------
+        layer : str
+            Layer to access.
+
+        sublayers : list of str
+            Sublayers to access.
+
+        Returns
+        -------
+        numpy.array
+            [sublayers x 2] array of start and stop indices for each
+            sublayer.
+        """
         if not isinstance(sublayers, list):
             sublayers = [sublayers]
         ind_list = []
@@ -348,22 +425,6 @@ class Network(object):
             ind_list.append(self.get_sublayer(layer, sublayer))
         return np.array(ind_list, dtype=np.dtype('i'))
 
-    def get_region(self, f_segment, c_segment):
-        """
-        Return slices for a region.
-
-        Returns
-        -------
-        f_ind : slice
-            Span of the region in the item dimension.
-
-        c_ind : slice
-            Span of the region in the context dimension.
-        """
-        f_ind = slice(*tuple(self.f_ind.get_segment(*f_segment)))
-        c_ind = slice(*tuple(self.c_ind.get_segment(*c_segment)))
-        return f_ind, c_ind
-
     def get_segment(self, layer, sublayer, segment):
         """
         Get indices for a segment.
@@ -423,6 +484,58 @@ class Network(object):
             raise ValueError(f'Invalid layer: {layer}')
         return ind
 
+    def get_slice(self, layer, sublayer, segment):
+        """
+        Get a slice for a segment.
+
+        Parameters
+        ----------
+        layer : {'f', 'c'}
+            Layer to access.
+
+        sublayer : str
+            Sublayer to access.
+
+        segment : str
+            Segment to access.
+
+        Returns
+        -------
+        slice
+            Slice for indexing the segment.
+        """
+        if layer == 'f':
+            index = self.f_ind
+        elif layer == 'c':
+            index = self.c_ind
+        else:
+            raise ValueError(f'Invalid layer: {layer}')
+        return index.get_slice(sublayer, segment)
+
+    def get_region(self, f_segment, c_segment):
+        """
+        Return slices for a region.
+
+        Parameters
+        ----------
+        f_segment : tuple of (str, str)
+            Sublayer and segment of the item layer.
+
+        c_segment : tuple of (str, str)
+            Sublayer and segment of the context layer.
+
+        Returns
+        -------
+        f_ind : slice
+            Span of the region in the item dimension.
+
+        c_ind : slice
+            Span of the region in the context dimension.
+        """
+        f_ind = self.f_ind.get_slice(*f_segment)
+        c_ind = self.c_ind.get_slice(*c_segment)
+        return f_ind, c_ind
+
     def add_pre_weights(self, connect, f_segment, c_segment, weights,
                         slope=1, intercept=0):
         """
@@ -673,7 +786,8 @@ class Network(object):
             distract_ind, param['distract_B']
         )
 
-    def record_study(self, segment, item_list, sublayers, B, Lfc, Lcf):
+    def record_study(self, segment, item_list, sublayers, B, Lfc, Lcf,
+                     include=None, exclude=None):
         """
         Study a list of items and record network states.
 
@@ -699,6 +813,13 @@ class Network(object):
         Lcf : float or numpy.array
             Learning rate for context to item associations.
 
+        include : list of str, optional
+            Network attributes to include in the recorded states.
+            Default is to include all attributes.
+
+        exclude : list of str, optional
+            Network attributes to exclude from the recorded states.
+
         Returns
         -------
         state : list of cymr.network.Network
@@ -713,10 +834,11 @@ class Network(object):
             self.present(
                 item, sublayers, param['B'][i], param['Lfc'][i], param['Lcf'][i]
             )
-            state.append(self.copy())
+            state.append(self.copy(include=include, exclude=exclude))
         return state
 
-    def record_recall(self, segment, recalls, sublayers, B, T, amin=0.000001):
+    def record_recall(self, segment, recalls, sublayers, B, T, amin=0.000001,
+                      include=None, exclude=None):
         """
         Simulate a recall sequence and record network states.
 
@@ -742,6 +864,13 @@ class Network(object):
             Minimum activation for each not-yet-recalled item on each
             recall attempt.
 
+        include : list of str, optional
+            Network attributes to include in the recorded states.
+            Default is to include all attributes.
+
+        exclude : list of str, optional
+            Network attributes to exclude from the recorded states.
+
         Returns
         -------
         state : list of cymr.network.Network
@@ -755,25 +884,25 @@ class Network(object):
         param = prepare_recall_param(n_item, n_sub, B, T, amin)
 
         f_ind = self.get_segment('f', *segment)
-        exclude = np.zeros(self.n_f, dtype=np.dtype('i'))
+        exclude_items = np.zeros(self.n_f, dtype=np.dtype('i'))
         state = []
         for i, recall in enumerate(recalls):
             # calculate item support
             operations.cue_item(
                 f_ind[0], n_item, self.w_cf_pre, self.w_cf_exp, self.w_ff_pre,
-                self.w_ff_exp, self.f_in, self.c, exclude, recalls, i
+                self.w_ff_exp, self.f_in, self.c, exclude_items, recalls, i
             )
             operations.apply_softmax(f_ind[0], n_item, self.f_in,
-                                     exclude, amin, param['T'])
+                                     exclude_items, amin, param['T'])
 
             # update the item layer
             ind = f_ind[0] + recall
             self.f[:] = 0
             self.f[ind] = 1
-            exclude[ind] = 1
+            exclude_items[ind] = 1
 
             # save context cue and the item it cued
-            state.append(self.copy())
+            state.append(self.copy(include=include, exclude=exclude))
 
             # update context
             item = (*segment, recall)
